"""
The :mod:`pyfan.aws.s3.pushsync` savse files to s3 and syncs

Includes method :func:`ar_draw_random_normal`.
"""

import os
import pyfan.aws.general.credentials as aws_credentials


def s3_upload(spn_img_pwdfn, st_s3_bucket='fans3testbucket', srt_s3_bucket_folder='pyfan_scatterline3'):
    """Upload an existing file to s3

    Upload to a particular bucket and subfolder, file in some local directory

    Parameters
    ----------
    spn_img_pwdfn: `string`
        full path to image, including the image name
    st_s3_bucket: `string`, optional
        Assuming that AWS credentials are already stored in the container on EC2
        or locally in a .aws credential file. So `st_s3_bucket` bucket name refers
        to bucket in the credentialed user's s3 account.
    srt_s3_bucket_folder: `string`, optional
        folder in s3 bucket to store image

    Returns
    -------
    none

    Examples
    --------
    >>> spn_img_pwdfn = 'C:/Users/fan/Downloads/data/test/test_image.png'
    >>> st_s3_bucket = 'fans3testbucket'
    >>> srt_s3_bucket_folder = 'pyfan_scatterline3/folder1/'
    >>> s3_upload(spn_img_pwdfn, st_s3_bucket, srt_s3_bucket_folder)
    """

    # s3 client
    s3 = aws_credentials.boto3_start_service(st_aws_service='s3')

    # Get file Path and the file name
    srt_path, snm_file = os.path.split(spn_img_pwdfn)
    spn_s3_bucket_folder_file = os.path.join(srt_s3_bucket_folder, snm_file).replace("\\", "/")

    # Uploading
    s3.upload_file(spn_img_pwdfn, st_s3_bucket, spn_s3_bucket_folder_file)



"""
The :mod:`pyfan.util.path.getfiles` generate and get various file paths.

Includes method :func:`gen_path_file`, :func:`gen_path`
"""

import os as os
from pathlib import Path
import pyfan.util.inout.iosupport as pyfan_iosup


def gen_path_file(spt_folder='', snm_file='', st_file_type=''):
    """Return full path to file given folder path and file name with suffix

    Parameters
    ----------
    spt_folder : str
        full path to folder
    snm_file : str
        file name without suffix (or with)
    st_file_type : str
        type of file, see options below, with pre-determined suffix, by default, there is no suffix. so if specify
        nothing, will save the `snm_file` with what is externally fed in

    Returns
    -------
    str
        Full path to file with path and file name.

    Examples
    --------
    >>> gen_path_file(spt_folder = 'C:/Users/fan/logvig/parameters/combo_type/',
    ...               snm_file = 'fs_gen_combo_type_20201030', st_file_type='log')
    C:\\Users\\fan\\logvig\\parameters\\combo_type\\fs_gen_combo_type_20201030.log.py
    """
    st_suffix = ''
    if st_file_type == 'log':
        st_suffix = '.log.py'

    return os.path.join(spt_folder, snm_file + st_suffix)


def gen_path(spt_root='', main_folder_name='', sub_folder_name=None, subsub_folder_name=None):
    """Generate a file path and return it

    Parameters
    ----------
    spt_root : str
        root name
    main_folder_name : str
        folder name
    sub_folder_name : str, optional
        subfolder name
    subsub_folder_name : str
        subsub folder name

    Returns
    -------
    str
        the full path to the folder

    Examples
    --------
    >>> gen_path(spt_root = 'C:/Users/fan/',
    ...          main_folder_name='logvig', sub_folder_name='parameters',
    ...          subsub_folder_name='combo_type')
    C:\\Users\\fan\\logvig\\parameters\\combo_type\\
    """
    spn_path_full = os.path.join(spt_root, main_folder_name, '')

    if sub_folder_name is not None:
        spn_path_full = pyfan_iosup.csvIO().createDirectory(spn_path_full, sub_folder_name)
        if subsub_folder_name is not None:
            spn_path_full = pyfan_iosup.csvIO().createDirectory(spn_path_full, subsub_folder_name)

    return spn_path_full


def fp_search_rglob(spt_root="../",
                    st_rglob='*.py',
                    ls_srt_subfolders=None,
                    verbose=False,
                    ):
    """Searches for files with search string in a list of folders

    Parameters
    ----------
    spt_root : string
        root folder to search in
    st_rglob : string
        search for files with this rglob string
    ls_srt_subfolders : :obj:`list` of :obj:`str`
        a list of subfolders of the spt_root to search in
    verbose: bool
        print details

    Returns
    -------
    :obj:`list` of :obj:`str`
        A list of file names

    Examples
    --------

    >>> ls_spn = fp_search_rglob(spt_root="../",
    >>>                          ls_srt_subfolders=['rmd', 'pdf'],
    >>>                          st_rglob = '*d*.py')
    [WindowsPath('../rmd/bookdownparse.py'), WindowsPath('../rmd/mattexmd.py'), WindowsPath('../rmd/rmdparse.py'), WindowsPath('../pdf/pdfgen.py')]

    """

    # Directories to search in
    if ls_srt_subfolders is None:
        # ls_srt_subfolders = ['calconevar/', 'derivative/', 'derivative_application/',
        #                      'matrix_basics/',
        #                      'opti_firm_constrained/', 'opti_hh_constrained_brsv/',
        #                      'opti_hh_constrained_brsv_inequality/']
        # ls_srt_subfolders = ['matrix_basics/']
        ls_spt = [spt_root]
    else:
        ls_spt = [spt_root + srt_subf for srt_subf in ls_srt_subfolders]

    if verbose:
        print(ls_spt)

    # get file names
    ls_spn_found_tex = [spn_file for spt_srh in ls_spt
                        for spn_file in Path(spt_srh).rglob(st_rglob)]
    if verbose:
        print(ls_spn_found_tex)

    return ls_spn_found_tex


# if __name__ == '__main__':
#     spt_root_u = "../"
#     ls_srt_subfolders_u = ['rmd', 'pdf']
#     st_rglob_u = '*d*.py'
#     verbose_u = False
#     ls_spn = fp_search_rglob(spt_root=spt_root_u, ls_srt_subfolders=ls_srt_subfolders_u,
#                              st_rglob=st_rglob_u, verbose=verbose_u)
#     print(ls_spn)



"""
The :mod:`pyfan.aws.general.path` file paths etc

Includes method :func:`detect_store_path`, :func:`save_img`.
"""

import platform as platform
from pathlib import Path
import os
import pyfan.aws.s3.pushsync as s3_pushsync


def detect_store_path(bl_check_path_exist=True, srt_sub_path=None, st_local_path=None):
    """Detects checks if program is running on an AWS Linux Instance

    In our case, all code that run on AWS linux are running inside conda containers.
    If running on container, save to data folder. If running on some local machine
    save results to the user's home path's download folder, data subfolder.

    Parameters
    ----------
    bl_check_path_exist : `bool`
        checking saving path if it does not exist
    srt_sub_path: `string`, optional
        this is the subpath to be used, in the data folder in EC2 container,
        or inside the downloads data folder under user directory.
    st_local_path: `string`, optional
        local overriding string save path, if not, use download/data folder. This
        will replace the local path

    Returns
    -------
    tuple[bool, string]
        returns boolean if on amzn splatform, then the directory where to store save files
    """

    # detect platform
    st_plotform = platform.release()

    # platform specific path
    if 'amzn' in st_plotform:
        amzn_linux_status = True
        spt_local_directory = '/data/'
        if srt_sub_path is not None:
            spt_local_directory = os.path.join(spt_local_directory, srt_sub_path)
    else:
        amzn_linux_status = False
        if st_local_path is None:
            spt_local_directory = os.path.join(str(Path.home()), 'Downloads', 'data')
            if srt_sub_path is not None:
                spt_local_directory = os.path.join(spt_local_directory, srt_sub_path)
        else:
            spt_local_directory = st_local_path

    # generate path if it does not exist
    if bl_check_path_exist:
        Path(spt_local_directory).mkdir(parents=True, exist_ok=True)

    return amzn_linux_status, spt_local_directory


def save_img(plt, sna_image_name, spt_image_path=None,
             dpi=300, papertype='a4',
             orientation='horizontal',
             bl_upload_s3=False, st_s3_bucket=None, srt_s3_bucket_folder=None):
    """Saves Graph Locally, and also upload to S3 if requested

    Given figure object,

    Parameters
    ----------
    plt: `matplotlib.pyplot`
        a matplotlib pyplot object from a graph that was just generated
    sna_image_name: `string`
        image name, without the suffix of png
    spt_image_path: `string`, optional
        path to image, if None, then use default local path in :func:`detect_store_path`
    dpi: `integer`, optional
        image dpi
    papertype: `string`, optional
        One of 'letter', 'legal', 'executive', 'ledger', 'a0' through 'a10', 'b0' through 'b10'.
    orientation: `string`, optional
        'horizontal' or 'portrait'
    bl_upload_s3: `bool`, optional
        if file will be uploaded to s3
    st_s3_bucket: `string`, optional
        Assuming that AWS credentials are already stored in the container on EC2
        or locally in a .aws credential file. So `st_s3_bucket` bucket name refers
        to bucket in the credentialed user's s3 account.
    srt_s3_bucket_folder: `string`, optional
        folder in s3 bucket to store image

    Returns
    -------
    tuple[bool, string]
        returns boolean if on amzn splatform, then the directory where to store save files
    """

    # Get Image Path, locally or locally in Ec2 Container
    amzn_linux_status, spt_local_directory = detect_store_path(bl_check_path_exist=True,
                                                               srt_sub_path=srt_s3_bucket_folder,
                                                               st_local_path=spt_image_path)

    # Save image locally
    snm_image_name = sna_image_name + '.png'
    spn_img_pwdfn = os.path.join(spt_local_directory, snm_image_name)
    plt.savefig(spn_img_pwdfn, dpi=dpi, papertype=papertype, orientation=orientation)

    # Given locally saved image copy over to aws
    if bl_upload_s3:
        s3_pushsync.s3_upload(spn_img_pwdfn, st_s3_bucket, srt_s3_bucket_folder)



# list update
dc_invoke_main_args_default = {'speckey': 'ng_s_t',
                               'ge': False,
                               'multiprocess': False,
                               'estimate': False,
                               'graph_panda_list_name': 'min_graphs',
                               'save_directory_main': 'simu',
                               'log_file': False,
                               'log_file_suffix': ''}
dc_invoke_main_args = dc_invoke_main_args_default
dc_invoke_main_args['speckey'] = 'b_ge_s_t_bis'
dc_invoke_main_args['ge'] = True
print(f'speckey in dc_invoke_main_args is {dc_invoke_main_args["speckey"]}.')
print(f'speckey in dc_invoke_main_args_default is {dc_invoke_main_args_default["speckey"]}.')

# raise error
ls_st_spec_key_dict = ['NG_S_D', 'NG_S_D=KAP_M0_NLD_M_SIMU=2=3']
st_connector = '='
ls_st_esti_simu = ['esti', 'simu']
for st_spec_key_dict in ls_st_spec_key_dict:
    for st_esti_simu in ls_st_esti_simu:
        it_len_spec_key_split = st_spec_key_dict.split(st_connector)
        if st_esti_simu == 'simu':
            if it_len_spec_key_split == 1:
                print('simulate with ' + st_spec_key_dict)
            else:
                st_error = 'st_spec_key_dict=' + st_spec_key_dict + ' can n'
                raise ValueError(st_error)



import boto3
import platform as platform
import yaml
from os import path


def boto3_start_service(st_aws_service='s3'):
    st_plotform = platform.release()

    # platform specific path
    if 'amzn' not in st_plotform:

        snm_aws_yml = "C:/Users/fan/fanwangecon.github.io/_data/aws.yml"
        if path.isfile(snm_aws_yml):
            fl_yaml = open(snm_aws_yml)
            ls_dict_yml = yaml.load(fl_yaml, Loader=yaml.BaseLoader)
            # Get the first element of the yml list of dicts
            aws_yml_dict_yml = ls_dict_yml[0]

            # Use AWS Personal Access Keys etc to start boto3 client
            aws_service = boto3.client(st_aws_service,
                                       aws_access_key_id=aws_yml_dict_yml['aws_access_key_id'],
                                       aws_secret_access_key=aws_yml_dict_yml['aws_secret_access_key'],
                                       region_name=aws_yml_dict_yml['region'])

    else:
        aws_service = boto3.client(st_aws_service)

    return aws_service



"""
The :mod:`pyfan.stats.markov.transprobcheck` checks markov transition row sums.

A markov transition matrix where each row does not
sum up to 1 due to simulation errors. Check if the gap between 1 and the row
values are too big, and then normalize.

import pyfan.stats.markov.transprobcheck as pyfan_stats_transprobcheck

Includes method :func:`markov_trans_prob_check` and :func:`markov_condi_prob2one`.
"""

import numpy as np


def markov_trans_prob_check(mt_trans, fl_atol_per_row=1e-05, fl_atol_avg_row=1e-08, fl_sum_to_match=1):
    """Markov conditional transition probability check

    Parameters
    ----------
    mt_trans : numpy.array of shape (N, N)
        The AR1 transition matrix, each row is a state, each value in each row is the conditional
        probability of moving from state i (row) to state j (column)
    fl_atol_per_row : `float`, optional
        Tolerance for the difference between 1 and each row sum
    fl_atol_avg_row : `float`, optional
        Tolerance for the difference between 1 and average of row sums
    fl_sum_to_match : `float`, optional
        This should be 1, unless the function is not used to handle transition matrixes
    Returns
    -------
    tuple
        A tuple of booleans, the first element is if satisfies the overall criteria. Second
        is if satisifes the per_row condition. Third if satisfies the average criteria.

    Examples
    --------
    >>> mt_ar1_trans = np.array([[0.4334, 0.5183, 0.0454],
    >>>                          [0.2624, 0.5967, 0.1245],
    >>>                          [0.1673, 0.5918, 0.2005]])
    >>> bl_ar1_sum_pass, bl_per_row_pass, bl_avg_row_pass = markov_trans_prob_check(mt_ar1_trans)
    >>> print(f'{bl_ar1_sum_pass=}')
    bl_ar1_sum_pass=False
    >>> print(f'{bl_per_row_pass=}')
    bl_per_row_pass=False
    >>> print(f'{bl_avg_row_pass=}')
    bl_avg_row_pass=False
    """

    ar_row_sums_ar1 = np.sum(mt_trans, axis=1)
    # per-row check
    bl_per_row_pass = np.allclose(ar_row_sums_ar1, fl_sum_to_match,
                                  rtol=0,
                                  atol=fl_atol_per_row)
    # all-rows check
    fl_row_gap_mean = np.mean(abs(ar_row_sums_ar1 - fl_sum_to_match))
    bl_avg_row_pass = np.allclose(fl_row_gap_mean + fl_sum_to_match, fl_sum_to_match,
                                  rtol=0,
                                  atol=fl_atol_avg_row)
    # Joint
    bl_ar1_sum_pass = bl_per_row_pass and bl_avg_row_pass

    return bl_ar1_sum_pass, bl_per_row_pass, bl_avg_row_pass


def markov_condi_prob2one(mt_trans):
    """Rescale markov transitions rows to sum to 1

    Suppose each transition matrix row sums up to slighly less than one, rescale so it sums to one.

    Parameters
    ----------
    mt_trans : numpy.array of shape (N, N)
        The AR1 transition matrix, each row is a state, each value in each row is the conditional
        probability of moving from state i (row) to state j (column)

    Returns
    -------
    ndarray
        The rescaled numpy array
    """

    # current row sums
    ar_row_sums_ar1 = np.sum(mt_trans, axis=1)
    ar_row_sums_ar1 = np.reshape(ar_row_sums_ar1, [-1, 1])
    # Update row values
    mt_ar1_trans_norm = mt_trans / np.reshape(ar_row_sums_ar1, [-1, 1])
    # ar_row_sums_ar1_norm = np.sum(mt_ar1_trans_norm, axis=1)
    # ar_row_sums_ar1_norm = np.reshape(ar_row_sums_ar1_norm, [-1, 1])

    return mt_ar1_trans_norm



"""
The :mod:`pyfan.graph.example.scatterline3` generates a graprh with three lines.
This is the functionalized vesrion of `plot_randgrid Example <https://pyfan.readthedocs.io/en/latest/auto_examples/plot_randgrid.html#sphx-glr-auto-examples-plot-randgrid-py>`_.

Includes method :func:`gph_scatter_line_rand`.
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import pyfan.gen.rand.randgrid as pyfan_gen_rand
import pyfan.aws.general.path as pyfan_path
import pyfan.util.timer.timer as pyfan_timer
import argparse
import sys
import os


# Parse Inputs to be used commandline
parser = argparse.ArgumentParser()
parser.add_argument('-A', dest="st_s3_bucket", help="s3 bucket to store output images", default='fans3testbucket')
parser.add_argument('-B', dest="it_seed", help="random seed", type=int, default=666)

args = parser.parse_args()


def gph_scatter_line_rand(fl_mu=0, fl_sd=1,
                          it_draws=25, it_seed=123,
                          fl_lower_sd=-2, fl_higher_sd=2,
                          bl_show_fig=True, bl_save_fig=False,
                          st_s3_bucket='fans3testbucket'):
    """A randomly generated graph with scatter plot and lines.

    Parameters
    ----------
    fl_mu, fl_sd : `float`, optional
        The mean and standard deviation of the normal process for lines
    it_draws: `integer`, optional
        Number of Draws lines
    it_seed: `integer`, optional
        External random seed externally. Default is 123. for lines
    fl_lower_sd, fl_higher_sd : `float`, optional
        Impose lower and upper bounds (in sd units) on shock draws. The normal
        distribution does not have lower or upper bounds.
    bl_show_fig: `bool`, optional
        Show graph in documentation if needed. When storing graph to disc and uploading
        to s3, do not need to show.

    Returns
    -------
    pandas.DataFrame of shape (`it_draws`, 4)
        A pandas dataframe with `it_draws` number of rows and four columns. First
        for x values, the next three for three types of randomly generated variables
        that are been plotted out.

    Examples
    --------
    >>> fl_mu = 0
    >>> fl_sd = 1
    >>> it_draws = 20
    >>> it_seed = 456
    >>> fl_lower_sd = -1
    >>> fl_higher_sd = 0.8
    >>> scatter_line_rand_graph(fl_mu, fl_sd,
	...						    it_draws, it_seed,
	...			                fl_lower_sd, fl_higher_sd)
		   x    shk_t0    shk_t1    shk_t2
	1    1.0 -0.668129 -2.000000 -2.000000
	2    2.0 -0.498210 -1.533950 -1.130231
	3    3.0  0.618576 -1.268601 -1.111846
	4    4.0  0.568692 -1.071098 -0.971485
	5    5.0  1.350509 -0.908400 -0.668129
	6    6.0  1.629589 -0.766786 -0.498210
	7    7.0  0.301966 -0.639112 -0.384060
	8    8.0  0.449483 -0.521108 -0.345811
	9    9.0 -0.345811 -0.409963 -0.325130
	10  10.0 -0.315231 -0.303676 -0.315231
	11  11.0 -2.000000 -0.200721 -0.106208
	12  12.0 -1.130231 -0.099856 -0.088752
	13  13.0 -1.111846  0.000000  0.237851
	14  14.0  0.237851  0.099856  0.301966
	15  15.0 -0.325130  0.200721  0.449483
	16  16.0  1.944702  0.303676  0.568692
	17  17.0  1.915676  0.409963  0.618576
	18  18.0  0.920348  0.521108  0.920348
	19  19.0  0.936398  0.639112  0.936398
	20  20.0  1.157552  0.766786  1.139873
	21  21.0 -0.106208  0.908400  1.157552
	22  22.0 -0.088752  1.071098  1.350509
	23  23.0 -0.971485  1.268601  1.629589
	24  24.0 -0.384060  1.533950  1.915676
	25  25.0  1.139873  2.000000  1.944702
    """

    # Type 0 Shock draw
    it_draw_type = 0
    ar_shock_t0 = \
        pyfan_gen_rand.ar_draw_random_normal(fl_mu, fl_sd, it_draws,
                                             it_seed, it_draw_type,
                                             fl_lower_sd, fl_higher_sd)

    # Type 1 Shock draw
    it_draw_type = 1
    ar_shock_t1 = \
        pyfan_gen_rand.ar_draw_random_normal(fl_mu, fl_sd, it_draws,
                                             it_seed, it_draw_type,
                                             fl_lower_sd, fl_higher_sd)

    # Type 2 Shock draw
    it_draw_type = 2
    ar_shock_t2 = \
        pyfan_gen_rand.ar_draw_random_normal(fl_mu, fl_sd, it_draws,
                                             it_seed, it_draw_type,
                                             fl_lower_sd, fl_higher_sd)

    # Draw Shocks Jointly
    fig, ax = plt.subplots()
    # Graph
    ar_it_x_grid = np.arange(1, it_draws + 1)
    ax.plot(ar_it_x_grid, ar_shock_t0,
            color='blue', linestyle='dashed', marker='x',
            label='Type 0: Bounded Shock Draws')
    ax.scatter(ar_it_x_grid, ar_shock_t1,
               color='red',
               label='Type 1: Quantile Points')
    ax.plot(ar_it_x_grid, ar_shock_t2,
            color='black', marker='d',
            label='Type 3: Sorted Bounded Shock Draws')
    # Labeling
    ax.legend(loc='upper left')
    plt.ylabel('Shock Values')
    plt.xlabel('Shock Draw Points')
    plt.title('Shock, Sorted and Bounded Shocks, Quantile Points')
    plt.grid()
    if bl_show_fig:
        plt.show()
    if bl_save_fig:
        sna_image_name = 'f_' + pyfan_timer.getDateTime(8) +'_s' + str(it_seed)

        srt_s3_bucket_folder = 'pyfan_gph_scatter_line_rand'
        pyfan_path.save_img(plt, sna_image_name,
                            dpi=300, papertype='a4',
                            orientation='horizontal',
                            bl_upload_s3=True, st_s3_bucket=st_s3_bucket,
                            srt_s3_bucket_folder=srt_s3_bucket_folder)
    # %%
    # Upload a local image
    # ------------------------

    mt_shocks = np.column_stack([ar_it_x_grid, ar_shock_t0, ar_shock_t1, ar_shock_t2])
    df_shocks = pd.DataFrame(data=mt_shocks,
                             index=range(1, mt_shocks.shape[0] + 1),
                             columns=['x', 'shk_t0', 'shk_t1', 'shk_t2'])
    return df_shocks


if __name__ == "__main__":

    # Run on command line, might need to install latest file locally first
    # conda activate base
    # cd "C:/Users/fan/pyfan/"
    # python setup.py install --user
    # python C:/Users/fan/pyfan/pyfan/graph/exa/scatterline3.py -A fans3testbucket -B 1
    # python /pyfan/pyfan/graph/exa/scatterline3.py -A fans3testbucket -B 1

    # This is an AWS Batch run with Job Array Index for Parallel processing
    # With this, only one job needs to be specified
    if "AWS_BATCH_JOB_ARRAY_INDEX" in os.environ:
        print('AWS_BATCH_JOB_ARRAY_INDEX')
        it_seed_arg = os.environ['AWS_BATCH_JOB_ARRAY_INDEX']
        it_seed_arg = int(it_seed_arg)
    else:
        it_seed_arg = args.it_seed

    print(it_seed_arg)

    gph_scatter_line_rand(fl_mu=0, fl_sd=1,
                          it_draws=25, it_seed=it_seed_arg,
                          fl_lower_sd=-2, fl_higher_sd=2,
                          bl_show_fig=False, bl_save_fig=True,
                          st_s3_bucket=args.st_s3_bucket)



"""
The :mod:`pyfan.devel.flog.logsupport` initiates logging and set logging options, output log path
points.

This is imported into other programs as *import pyfan.devel.flog.logsupport as pyfan_logsup*

Includes method :func:`log_vig_start`, :func:`log_format`
"""

import logging
import pyfan.util.path.getfiles as pyfan_getfiles
import pyfan.util.timer.timer as pyfan_timer
import numpy as np


def log_vig_start(spt_root, main_folder_name, file_name='fs_gen_combo_type',
                  sub_folder_name=None, subsub_folder_name=None,
                  it_time_format=8, log_level=logging.WARNING,
                  **kwargs):
    """Start logging to log file

    Generate path to log file and initiate log file. Return this full path to log file. Configure the log file
    with formating

    Parameters
    ----------
    spt_root : str
        folder root to log file.
    main_folder_name : str
        main folder, appended to `spt_root`.
    file_name : str
        file name for the log file, without suffix.
    sub_folder_name : str, optional
        possible subfolder name. This is double pound vig level.
    subsub_folder_name : str, optional
        possible subsub folder name. try not to have lower than this level. This is triple pound vig level.
    it_time_format : int
        different types of time formatting, if `it_time_format` is zero, no time suffix
    log_level : int
        logging level integers to report, including CRITICAL 50 ERROR 40 WARNING 30 INFO 20 DEBUG 10 NOTSET 0.
    **kwargs
        Arguments for functions that is called, including :func:`log_format`

    Returns
    -------
    str
        return the path to the log file

    Examples
    --------
    >>> log_vig_start(spt_root = 'C:/Users/fan/',
    ...               main_folder_name='logvig', sub_folder_name='parameters',
    ...               subsub_folder_name='combo_type',
    ...               file_name='fs_gen_combo_type',
    ...               it_time_format=8, log_level=logging.INFO)
    C:\\Users\\fan\\logvig\\parameters\\combo_type\\fs_gen_combo_type_20201030.log.py
    """

    # A. Generate path
    spt_log = pyfan_getfiles.gen_path(spt_root, main_folder_name, sub_folder_name, subsub_folder_name)

    # B. File name with suffix
    if it_time_format == 0:
        snm_log = file_name
    else:
        snm_log = file_name + '_' + str(pyfan_timer.getDateTime(it_time_format))

    # C. Generate full path to log file
    spn_log = pyfan_getfiles.gen_path_file(spt_log, snm_log, st_file_type='log')

    # D. Start logging using the log file
    logging.basicConfig(filename=spn_log, filemode='w', level=log_level, format=log_format(**kwargs))

    return spn_log


def log_format(bl_set_print_opt=True, it_print_opt=1):
    """Logging formats

    This is called by :func:`log_vig_start`, with parameters fed in with *kwargs*

    Parameters
    ----------
    bl_set_print_opt : bool, optional
        If to set numpy table printing options, how many columns and decimal controls
    it_print_opt : int, optional
        Different possible options to set

    Returns
    -------
    str
        formatting string options for logging config

    Examples
    --------
    >>> log_format(bl_set_print_opt = True, it_print_opt = 1)
    '%(filename)s - %(funcName)s - %(lineno)d -  %(asctime)s - %(levelname)s %(message)s'
    """
    if bl_set_print_opt:
        if it_print_opt == 1:
            np.set_printoptions(precision=2, linewidth=100, suppress=True, threshold=3000)

    return '%(filename)s - %(funcName)s - %(lineno)d -  %(asctime)s - %(levelname)s %(message)s'
